#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <vector>
#include <fstream>
#include <iostream>
#include <algorithm>
#include <string>
#include <utility>


#include <cuda.h>
#include "util.h"
#include "PSO.h"
#include "NN.h"

using namespace std;



__device__ __host__ float sigmoid(float x)
{
    return 1 / (1 + expf(-x));
}


/**  weights[0] == w0, weights[layer_size(not layer_size-1)] == wN
    1 neuron weights width == prev_layer_size+1 //refactor later?
    input width == previous_layer_size
    output width == layer_size
*/
__device__ __host__ void propagate_layer(float *input, float *weights, float *output,int previous_layer_size,int layer_size, int isLastLayer)
{
    for(int i=0;i<layer_size;i++)
    {
        float temp_output = weights[i*(previous_layer_size+1)];
        for(int j=0;j<previous_layer_size;j++)
        {
            temp_output += input[j]*weights[i*(previous_layer_size+1)+j+1];
        }
        if(!isLastLayer) temp_output = sigmoid(temp_output);
        output[i] = temp_output;
    }
}

/** Output gets stored in output which has to be preallocated
    output and temp_output has to be in fast/shared memory */
// output and temp_output = memory size of widest layer(except input layer), for functions use
// num_layers = (input + N*hidden + output)
// layers[i] - number of neurons in i-th layer
// in_ranges - input i will be scaled to in_range [[2*i],[2*i+1]]
// out_ranges - output i will be scaled to out_ranges [[2*i],[2*i+1]]
__device__ __host__ void simulate_neural_network(float *input, float *weights, float *output,int num_layers,int *layers, float *out_ranges, float *temp_output, int lastLayerLinear)
{ //ljepsa verzija je sporija
    int isLastLayer=0;
    if(num_layers == 2) isLastLayer = 1;
    if(num_layers%2==0) propagate_layer(input, weights, output,layers[0],layers[1],isLastLayer);
    else propagate_layer(input, weights, temp_output,layers[0],layers[1],isLastLayer);
    int weights_index = layers[1]*(layers[0]+1);

    for(int i=2;i<num_layers;i++)
    {
        if(i == num_layers - 1 && lastLayerLinear) isLastLayer = 1;

        if((num_layers-i)%2==1) propagate_layer(temp_output, &weights[weights_index], output,layers[i-1],layers[i],isLastLayer);
        else propagate_layer(output, &weights[weights_index], temp_output,layers[i-1],layers[i],isLastLayer);
        weights_index += layers[i]*(layers[i-1]+1);
    }

    // now scale the output to [0,1]
    int outputSize = layers[num_layers-1];
    for(int i=0;i<outputSize;i++)
        output[i] = output[i] * (out_ranges[2*i+1] - out_ranges[2*i]) + out_ranges[2*i];

    //result is now in output
}

//number of weights for a given neural network configuration (including bias) //perhaps include in the class later
__host__ __device__ int num_weights(int num_layers,int *layers)
{
    int n = 0;
    for(int i=1;i<num_layers;i++)
        n += layers[i] * (layers[i-1]+1);
    return n;
}

__host__ __device__ int widest_layer(int num_layers, int *layers)
{
    int widest = 0;
    for(int i=1;i<num_layers;i++)
        widest = (layers[i]>widest)?layers[i]:widest;
    return widest;
}



/** simulates number_of_networks with number_of_inputs different inputs at once and returns outputs(first all outputs for the first network, then for the second...) */
template <int widest_layer>
__global__ void global_simulate_neural_networks_local_mem(float *inputs, float *weights, float *outputs, int num_layers,int *layers, float *ranges, int number_of_networks, int number_of_inputs, int num_weights_, int lastLayerLinear)
{
    int test_idx = blockIdx.y * blockDim.x + threadIdx.x;
    int nn_idx = blockIdx.x;
    //if(test_idx < number_of_inputs)
    if(test_idx < number_of_inputs && nn_idx < number_of_networks)
    {
        int input_size = layers[0];
        int output_size = layers[num_layers-1];

        float output[widest_layer]; //ovo treba bit dynamic
        float temp_output[widest_layer];

        simulate_neural_network(&inputs[test_idx*input_size], &weights[num_weights_*nn_idx], output, num_layers, layers, ranges, temp_output, lastLayerLinear);
        for(int i=0;i<output_size;i++)
            outputs[nn_idx*number_of_inputs*output_size+output_size*test_idx + i] = output[i];
    }
}

__global__ void device_compute_mse(float *ouputs, float *targets, float *MSEs, int number_of_networks, int num_test_samples, int output_size)
{
    int netIdx = blockIdx.x * blockDim.x + threadIdx.x;
    if(netIdx < number_of_networks)
    {
        float mse = 0;
        for(int i = 0; i<num_test_samples;i++)
        {
            for(int j = 0; j < output_size;j++)
            {
                float d = ouputs[netIdx*num_test_samples+i*output_size+j] - targets[i*output_size+j];
                mse += d * d;
            }
        }
        MSEs[netIdx] = mse / (num_test_samples * output_size);
    }
}


extern __shared__ float shared_memory[];
__global__ void global_simulate_neural_networks_shared_mem(float *inputs, float *weights, float *outputs, int num_layers,int *layers, float *out_ranges, int number_of_networks, int number_of_inputs, int num_weights_, int widest_layer_, int lastLayerLinear)
{
    int idx = threadIdx.x;

    // assign pointers to shared memory
    int *sh_layers = (int *)shared_memory;

    // load layers to shared memory
    for(int i=idx;i<num_layers;i+=blockDim.x)
        sh_layers[i] = layers[i];

    float *sh_weights = (float *)&sh_layers[num_layers];
    float *sh_out_ranges = (float *)&sh_weights[num_weights_];

    float *sh_output = (float *)&sh_out_ranges[sh_layers[num_layers-1]*2+idx*widest_layer_];
    float *sh_temp_output = (float *)&sh_output[blockDim.x*widest_layer_];


    int test_idx = blockIdx.y * blockDim.x + threadIdx.x;
    int nn_idx = blockIdx.x;

    // load ranges to shared memory
    for(int i=idx;i<sh_layers[num_layers-1]*2;i+=blockDim.x)
        sh_out_ranges[i] = out_ranges[i];

    // load weights to shared memory
    for(int i=idx;i<num_weights_;i+=blockDim.x)
        sh_weights[i] = weights[num_weights_*nn_idx+i];
    __syncthreads();

    //if(test_idx < number_of_inputs)
    if(test_idx < number_of_inputs && nn_idx < number_of_networks)
    {
        int input_size = sh_layers[0];
        int output_size = sh_layers[num_layers-1];

        simulate_neural_network(&inputs[test_idx*input_size], sh_weights, sh_output, num_layers, sh_layers, sh_out_ranges, sh_temp_output, lastLayerLinear);
        for(int i=0;i<output_size;i++)
            outputs[nn_idx*number_of_inputs*output_size+output_size*test_idx + i] = sh_output[i];
    }
}


// c++

/** Reads in network layer configuration from file to host vector h_layers_ */
void NeuralNetworks::readInNNetConfig(string fileName)
{
    ifstream file(fileName.c_str());
    if(!file.is_open()) error(string("Error opening file: ") + fileName, -3);

    //read layers from file to host vector
    int layer;
    while(file >> layer)
        h_layers_.push_back(layer);
}

/** Reads in training data from file to host vectors h_inputs_ and h_targets_ */
void NeuralNetworks::readInTrainingData(std::string fileName)
{
    ifstream file(fileName.c_str());
    if(!file.is_open()) error(string("Error opening file: ") + fileName, -7);

    // number of inputs and outputs
    int num_in, num_out;
    file >> num_in >> num_out;

    if(num_in != getInputSize()) error("Input size from config file does not match data input size", -149);
    if(num_out != getOutputSize()) error("Output size from config file does not match data output size", -149);

    // input and ouput ranges
    in_ranges_ = vector<float>(2*num_in);
    for(int i=0;i<num_in;i++)
        file >> in_ranges_[2*i] >> in_ranges_[2*i+1];

    h_out_ranges_ = vector<float>(2 * num_out);
    for(int i=0;i<num_out;i++)
        file >> h_out_ranges_[2*i] >> h_out_ranges_[2*i+1];

    // training samples
    float tmp_num;
    while(file >> tmp_num)
    {
        // read input
        h_inputs_.push_back(tmp_num);
        for(int i=1;i<num_in;i++)
        {
            file >> tmp_num;
            h_inputs_.push_back(tmp_num);
        }

        // read output
        for(int i=0;i<num_out;i++)
        {
            file >> tmp_num;
            h_targets_.push_back(tmp_num);
        }
    }
    h_inputs_ = scaleInputData(h_inputs_);
}

/** Scales several input data samples which are in ranges in_ranges_ to [-1,1]*/
vector<float> NeuralNetworks::scaleInputData(vector<float> in)
{
    for(size_t i=0;i<in.size() / getInputSize();i++)
        for(int j=0;j<getInputSize();j++)
        {
            //cout << "in before= " << in[i*getInputSize()+j];
            in[i*getInputSize()+j] = 2 * (in[i*getInputSize()+j] - in_ranges_[2*j]) / (in_ranges_[2*j+1] - in_ranges_[2*j]) - 1;
            //cout << "   in after= " << in[i*getInputSize()+j] << endl;
        }
    return in;
}

/** This is the main constructor. It receives names of files for network configuration ,training set and PSO parameters
    and allocates memory on host and device(for all the networks). All data that doesn't change gets copied to device memory.
    Only weights can get changed. */
NeuralNetworks::NeuralNetworks(string nnetConfigFile, string trainingDataFile, string psoConfigFile = string())
{
    readInNNetConfig(nnetConfigFile); //h_layers_
    readInTrainingData(trainingDataFile); // h_inputs_ , h_targets_
    if(!psoConfigFile.empty()) psoParameters_ = PsoParameters(psoConfigFile);

    psoParameters_.nDimensions = getNumWeights();

    lastLayerLinear_ = false;

    //set block size and memory type for global function
    useSharedMemoryInSimulation_ = true;
    blockSize_ = maxBlockSize();

    if(blockSize_ < 16) // neural network too big for share memory, use local memory
    {
        blockSize_ = 128;
        useSharedMemoryInSimulation_ = false;
    }
    if(blockSize_ > 128) blockSize_ = 128; //block doesn't have to be too big


    // allocate host memory
    h_weights_ = generate_random_vector(getNumWeights()*getNumberOfNetworks(),-1,1);
    h_outputs_ = vector<float>(h_targets_.size()*getNumberOfNetworks());

    // allocate memory for CPU simulation
    h_output_ = new float[getWidestLayer()];
    h_temp_output_ = new float[getWidestLayer()];

    // allocate device memory
    cudaMalloc((void**)&d_weights_,h_weights_.size()*sizeof(float));
    cudaMalloc((void**)&d_inputs_,h_inputs_.size()*sizeof(float));
    cudaMalloc((void**)&d_targets_,h_targets_.size()*sizeof(float));
    cudaMalloc((void**)&d_outputs_,getNumberOfNetworks()*h_targets_.size()*sizeof(float));
    cudaMalloc((void**)&d_out_ranges_, 2* getOutputSize() *sizeof(float));
    cudaMalloc((void**)&d_layers_,h_layers_.size()*sizeof(int));
    cudaMalloc((void**)&d_MSEs_,getNumberOfNetworks()*sizeof(float));
    if(d_weights_==NULL || d_inputs_==NULL || d_targets_==NULL || d_outputs_==NULL || d_out_ranges_==NULL || d_layers_==NULL || d_MSEs_==NULL)
        error("Error allocating device memory", -12);

    // copy data to device
    int status = 0;
    status |= cudaMemcpy(d_weights_, &h_weights_[0], h_weights_.size()*sizeof(float), cudaMemcpyHostToDevice);
    status |= cudaMemcpy(d_inputs_, &h_inputs_[0], h_inputs_.size()*sizeof(float), cudaMemcpyHostToDevice);
    status |= cudaMemcpy(d_targets_, &h_targets_[0], h_targets_.size()*sizeof(float), cudaMemcpyHostToDevice);
    status |= cudaMemcpy(d_out_ranges_, &h_out_ranges_[0], h_out_ranges_.size()*sizeof(float), cudaMemcpyHostToDevice);
    status |= cudaMemcpy(d_layers_, &h_layers_[0], h_layers_.size()*sizeof(int), cudaMemcpyHostToDevice);
    if(status != cudaSuccess) error("Error copying to device memory", -17);

}


/** Not for a training network, just for operator()().
    No device memory used, just host weights vector.*/
NeuralNetworks::NeuralNetworks(string weightsFile)
{
    ifstream file(weightsFile.c_str(), ios::in | ios::binary);
    if(!file.is_open()) error(string("Error opening file: ") + weightsFile, -38);

    psoParameters_.populationSize = 1;
    psoParameters_.nDimensions = getNumWeights();

    lastLayerLinear_ = false;

    // read numLayers
    int numLayers;
    file.read((char *)&numLayers, sizeof(int));

    // read layers
    h_layers_ = vector<int>(numLayers);
    file.read((char *)&h_layers_[0], numLayers * sizeof(int));

    // write weights
    h_weights_ = vector<float>(getNumWeights());
    file.read((char *)&h_weights_[0], getNumWeights() * sizeof(float));

    // write input and output ranges
    in_ranges_ = vector<float>(2*getInputSize());
    file.read((char *)&in_ranges_[0], 2*getInputSize() * sizeof(float));
    h_out_ranges_ = vector<float>(2*getOutputSize());
    file.read((char *)&h_out_ranges_[0], 2*getOutputSize() * sizeof(float));

    file.close();

    // allocate memory for CPU simulation
    h_output_ = new float[getWidestLayer()];
    h_temp_output_ = new float[getWidestLayer()];
}

NeuralNetworks::~NeuralNetworks()
{
    delete [] h_output_;
    delete [] h_temp_output_;

    cudaFree(d_weights_);
    cudaFree(d_inputs_);
    cudaFree(d_targets_);
    cudaFree(d_outputs_);
    cudaFree(d_out_ranges_);
    cudaFree(d_layers_);
    cudaFree(d_MSEs_+2);
}

void NeuralNetworks::updateWeights(const std::vector<float> &weights)
{
    if(h_weights_.size() != weights.size()) error("Wrong weights size.",-21);

    // update host weights
    h_weights_ = weights;

    // update device weights
    int status = cudaMemcpy(d_weights_, &h_weights_[0], getNumberOfNetworks()*getNumWeights()*sizeof(float), cudaMemcpyHostToDevice);
    if(status != cudaSuccess) error("Error copying to device memory", -23);
}

/** Saves everything neccessary to simulate the best network.
    h_weights_ of the best network,
    h_layers_,
    in_ranges, h_out_ranges
    */
void NeuralNetworks::saveWeights(std::string fileName)
{
    ofstream file(fileName.c_str(), ios::out | ios::binary);
    if(!file.is_open()) error(string("Error opening file: ") + fileName, -55);

    vector<float> mse = device_computeMSE();
    vector<float>::iterator best_mse_iter = min_element(mse.begin(),mse.end());
    int best_mse_index = best_mse_iter - mse.begin();

    // write numLayers
    int numLayers = getNumLayers();
    file.write((char *)&numLayers, sizeof(int));
    // write layers
    file.write((char *)&h_layers_[0], getNumLayers() * sizeof(int));

    // write weights
    file.write((char *)&h_weights_[best_mse_index * getNumWeights()], getNumWeights() * sizeof(float));

    // write input and output ranges
    file.write((char *)&in_ranges_[0], 2*getInputSize() * sizeof(float));
    file.write((char *)&h_out_ranges_[0], 2*getOutputSize() * sizeof(float));

    file.close();
}

vector<float>& NeuralNetworks::getWeights()
{
    return h_weights_;
}
std::vector<float>& NeuralNetworks::getOutputs()
{
    return h_outputs_;
}

void NeuralNetworks::printInfo()
{
    cout << "Neural network info" << endl << "-------------------" << endl;
    cout << "Number of test data: " << h_inputs_.size()/h_layers_[0] << endl;
    cout << "Layers: ";
    for(size_t i=0;i<h_layers_.size();i++) cout << h_layers_[i] << endl;
    cout << "Input ranges:\n";
    for(int i=0;i<getInputSize();i++) cout << "[" << in_ranges_[2*i] << "," << in_ranges_[2*i+1] << "]" << endl;
    cout << "Output ranges:\n";
    for(int i=0;i<getOutputSize();i++) cout << "[" << h_out_ranges_[2*i] << "," << h_out_ranges_[2*i+1] << "]" << endl;
    cout << "Total weights: " << getNumWeights() << endl;
    cout << blockSize_ << " threads per block." << endl;
    if(useSharedMemoryInSimulation_)
        cout << "Using shared memory for simulation function (" << sharedMemoryRequirements(blockSize_) << "B per block)." << endl;
    else cout << "Using local memory for simulation function." << endl;

    if(lastLayerLinear_) cout << "Last layer is linear" << endl;
    else cout << "Last layer is not linear" << endl;

    float average_diff;
    float max_diff;
    GPU_CPU_output_difference(average_diff,max_diff);
    cout << "Difference between CPU and GPU output: max=" << max_diff << " average=" << average_diff << endl;

    psoParameters_.PrintInfo();
    /*
    vector<float> host_mse = host_computeMSE();
    cout << "Mean sqared error: " << endl;
    for(size_t i=0; i<host_mse.size();i++)
    {
        cout << " " << i << ". " << host_mse[i] << endl;
    }*/


    /*
    cout << "Test data:" << endl;
    for(int i=0;i<getNumSamples();i++)
    {
        cout << "in:";
        for(int j=0;j<getInputSize();j++)
            cout << " " << h_inputs_[i*getInputSize()+j];
        cout << "  out:";
        for(int j=0;j<getOutputSize();j++)
            cout << " " << h_targets_[i*getOutputSize()+j];

        cout << endl;

    }*/
}

void NeuralNetworks::GPU_CPU_output_difference(float &average_diff,float &max_diff)
{
    host_SimulateNeuralNetworks();
    vector<float> host_vec = getOutputs();
    device_SimulateNeuralNetworks();
    vector<float> device_vec = getOutputs();
    average_diff = 0;
    max_diff = 0;
    for(size_t i=0; i<host_vec.size();i++)
    {
        float diff = fabs(host_vec[i] - device_vec[i]);
        max_diff = max((float)max_diff, (float)diff);
        average_diff += diff;
    }
    average_diff /= host_vec.size();
}

int NeuralNetworks::sharedMemoryRequirements(int blockSize, bool includingArguments)
{
    // kernel arguments get passed by shared memory
    int arguments_size = 5 * sizeof(float *) + 6 * sizeof(int) + sizeof(int *);
    arguments_size = 256; // ? maybe all 256 bytes are taken
    if(!includingArguments) arguments_size = 0;
    return (getNumWeights() + blockSize*2*getWidestLayer() + 2*getOutputSize()) * sizeof(float) + getNumLayers() * sizeof(int) + arguments_size;
}

int NeuralNetworks::maxBlockSize()
{
    cudaDeviceProp deviceProp;
    cudaGetDeviceProperties(&deviceProp, 0);
    int maxSharedMem = deviceProp.sharedMemPerBlock;

    int blockSize;
    for(blockSize=deviceProp.maxThreadsPerBlock;blockSize>0;blockSize--)
        if(sharedMemoryRequirements(blockSize, true) <= maxSharedMem) break;
    return blockSize;
}

/*
void NeuralNetworks::host_SimulateNeuralNetworks() //2.5 puta sporije
{
    int output_size = getOutputSize();
    int number_of_inputs =  getInputSize();
    int num_weights = getNumWeights();
    for(int i=0;i<number_of_networks_;i++)
    {
        for(size_t j=0;j<h_inputs_.size();j++)
        {
            simulate_neural_network(&h_inputs_[j*getInputSize()], &h_weights_[num_weights*i], h_output_,h_layers_.size(),&h_layers_[0], h_temp_output_);
            std::copy(h_output_, h_output_+output_size,&h_outputs_[i*number_of_inputs*output_size+output_size*j]);
        }
    }
}*/

void NeuralNetworks::host_SimulateNeuralNetworks()
{
    // ako se direktno koriste vectori i class varijable, onda je 2.5 puta sporije
    float *weights = &h_weights_[0];
    float *inputs = &h_inputs_[0];
    int number_of_inputs = h_inputs_.size()/getInputSize();
    int *layers = &h_layers_[0];
    float *out_ranges = &h_out_ranges_[0];
    int num_layers = getNumLayers();
    float *temp_output = h_temp_output_;
    float *output = h_output_;
    int output_size = getOutputSize();
    int num_weights = getNumWeights();
    int number_of_networks_ = getNumberOfNetworks();
    for(int i=0;i<number_of_networks_;i++)
    {
        for(int j=0;j<number_of_inputs;j++)
        {
            simulate_neural_network(&inputs[j*layers[0]], &weights[num_weights*i], output, num_layers, layers, out_ranges, temp_output, lastLayerLinear_);
            std::copy(output, output+output_size,&h_outputs_[i*number_of_inputs*output_size+output_size*j]);
        }
    }
}

//perhaps not copy output to host, just do fitness calculations on device
void NeuralNetworks::device_SimulateNeuralNetworks()
{
    // run kernel
    dim3 gridSize(getNumberOfNetworks(), (getNumSamples()+blockSize_-1)/blockSize_);
    if(useSharedMemoryInSimulation_)
    {
        int sharedMemorySize = sharedMemoryRequirements(blockSize_);
        global_simulate_neural_networks_shared_mem<<<gridSize,blockSize_,sharedMemorySize>>>(d_inputs_, d_weights_, d_outputs_, getNumLayers(),d_layers_, d_out_ranges_, getNumberOfNetworks(), getNumSamples(), getNumWeights(), getWidestLayer(), lastLayerLinear_);
    }
    else //use local memory
    {
        const int widest_layer = 1000;
        if(getWidestLayer() > widest_layer) error("Widest layer too big(set widest_layer constant to a higher value).",-29);
        global_simulate_neural_networks_local_mem<widest_layer><<<gridSize,blockSize_>>>(d_inputs_, d_weights_, d_outputs_, getNumLayers(),d_layers_, d_out_ranges_, getNumberOfNetworks(), getNumSamples(), getNumWeights(), lastLayerLinear_);
    }

    // copy output back to host
    cudaMemcpy(&h_outputs_[0], d_outputs_, getNumberOfNetworks()*getNumSamples()*getOutputSize()*sizeof(float), cudaMemcpyDeviceToHost);

}


void NeuralNetworks::device_SimulateNeuralNetworks_forceLocalMem()
{
    bool useShared = useSharedMemoryInSimulation_;

    useSharedMemoryInSimulation_ = false;

    device_SimulateNeuralNetworks();

    useSharedMemoryInSimulation_ = useShared;
}

std::vector<float> NeuralNetworks::operator() (std::vector<float> input)
{
    if((int)input.size() != getInputSize())
        error("Wrong number of inputs",-47);

    input = scaleInputData(input);

    int best_mse_index = 0;

    if(getNumberOfNetworks() > 1)
    {
        vector<float> mse = device_computeMSE();

        vector<float>::iterator best_mse_iter = min_element(mse.begin(),mse.end());
        best_mse_index = best_mse_iter - mse.begin();
    }

    vector<float> output(getOutputSize());

    simulate_neural_network(&input[0], &h_weights_[getNumWeights()*best_mse_index], h_output_,getNumLayers(),&h_layers_[0], &h_out_ranges_[0], h_temp_output_, lastLayerLinear_);
    std::copy(h_output_, h_output_+getOutputSize(),output.begin());

    return output;
}

vector<float> NeuralNetworks::host_computeMSE()
{
    vector<float> nMSE(getNumberOfNetworks());
    int output_size = getOutputSize();
    int num_samples = getNumSamples();
    int number_of_networks = getNumberOfNetworks();
    for(int i = 0; i < number_of_networks; i++)
    {
        float mse = 0;
        for(int j = 0; j < num_samples; j++)
        {
            //cout << " j =" << j << endl;
            for(int k = 0; k < output_size;k++)
            {
                float d = h_outputs_[(i*num_samples+j)*output_size+k] - h_targets_[j*output_size+k];
                mse += d * d;
            }
        }
        nMSE[i] = mse/(num_samples*output_size);
    }
    return nMSE;
}

vector<float> NeuralNetworks::device_computeMSE()
{
    vector<float> nMSE(getNumberOfNetworks());
    // one thread per network (not the most efficient, but doesn't matter much)
    int blockSize = 16;
    int gridSize = (int)ceil((float)getNumberOfNetworks()/blockSize);

    ::device_compute_mse<<<gridSize,blockSize>>>(d_outputs_, d_targets_, d_MSEs_, getNumberOfNetworks(), getNumSamples(), getOutputSize());
    cudaMemcpy(&nMSE[0], d_MSEs_, getNumberOfNetworks()*sizeof(float), cudaMemcpyDeviceToHost);

    return nMSE;
}


// If any of the conditions is 0, it is not checked.
// returns training time without the overhead (just the loop).
double NeuralNetworks::train(int forceSimulationType)
{



    if(psoParameters_.maxFitness == 0 && psoParameters_.iterations == 0 && psoParameters_.time == 0)
        error("No terminating conditions",-79);


    clock_t start = clock();
    clock_t end = start + CLOCKS_PER_SEC * psoParameters_.time;

    ParticleSwarmOptimization pso(psoParameters_);
    int i=-1;

    event_pair timer;

    while( ((psoParameters_.maxFitness == 0)?(1):(pso.getSwarmFitness() > psoParameters_.maxFitness)) &&
           ((psoParameters_.iterations == 0)?(1):(i < psoParameters_.iterations)) &&
           ((psoParameters_.time == 0)?(1):(clock()<end)) )
    {
        if(i == 0) // won't count the preprocessing
            start_timer(&timer);

        //start_timer(&timer);

        if(i > -1) // -1 == preprocessing, just for updating fitnesses
            pso.nextIteration();


        updateWeights(pso.getPositions());

        //start_timer(&timer);
        if(forceSimulationType == no_force || forceSimulationType == force_GPU_shared)
            device_SimulateNeuralNetworks();
        else if(forceSimulationType == force_CPU)
            host_SimulateNeuralNetworks();
        else if(forceSimulationType == force_GPU_local)
            device_SimulateNeuralNetworks_forceLocalMem();

        vector<float> mses = host_computeMSE();//device_computeMSE();

        pso.updateFitnesses(mses);
        i++;
        if(i%50==0)
        {
            cout << "Swarm fitness (" << i << ") = " << pso.getSwarmFitness() << endl;
            //cout << "mean speed = " << vector_mean(pso.particleSpeeds_) << " +- " << vector_stdev(pso.particleSpeeds_) << "    ";
            //cout << "mean pos = " << vector_mean(pso.getPositions()) << " +- " << vector_stdev(pso.getPositions()) << "   ";
            //cout << "max weight = " << vector_max(pso.getPositions()) << "    ";
            //cout << "abs speed mean = " << vector_abs_mean(pso.particleSpeeds_) << endl;
            //cout << "max error = " << maxError() << endl;
        }

    }
    check_cuda_error("PSO training");
    double PSO_time = elapsed_time(&timer);
    return PSO_time;
}



float NeuralNetworks::maxError()
{
    float maxE = -1;
    /* treba se modify-at da gleda sam najbolju
    for(size_t i=0; i<h_outputs_.size();i++)
        maxE = max(maxE,abs(h_outputs_[i]-h_targets_[i%getNumSamples()]));*/
    return maxE;
}

int NeuralNetworks::getInputSize()
{
    return h_layers_[0];
}

int NeuralNetworks::getOutputSize()
{
    return h_layers_.back();
}

int NeuralNetworks::getNumWeights()
{
    return num_weights(getNumLayers(), &h_layers_[0]);
}

int NeuralNetworks::getNumLayers()
{
    return h_layers_.size();
}
int NeuralNetworks::getWidestLayer()
{
    return widest_layer(getNumLayers(),&h_layers_[0]);
}
int NeuralNetworks::getNumSamples()
{
    return h_inputs_.size()/getInputSize();
}
int NeuralNetworks::getNumberOfNetworks()
{
    return psoParameters_.populationSize;
}

// ovo ispod je vise za mjerenje
int NeuralNetworks::getNumSimulations()
{
    return getNumberOfNetworks()*getNumSamples();
}

void NeuralNetworks::reduceNumSamples(int num)
{
    h_inputs_.resize(num*getInputSize());
    h_outputs_.resize(num*getOutputSize());
}

int NeuralNetworks::getThreadsPerBlock()
{
    return blockSize_;
}

int NeuralNetworks::getNumBlocks()
{
    return getNumberOfNetworks() * ((getNumSamples()+blockSize_-1)/blockSize_);
}

int NeuralNetworks::memSizeReadFromGlobal()
{
    int totalMem = getNumBlocks()*( getNumLayers() + getOutputSize() * 2 + getNumWeights() );
    totalMem += getNumSimulations()*( getInputSize() + getOutputSize() ); //words
    totalMem *= 4; // bytes
    return totalMem;
}
