#ifndef UTIL_H
#define UTIL_H

#include <string>
#include <vector>

#include <stdio.h>

void error(std::string str, int code=-1);

std::vector<float> generate_random_vector(int n, float lower_limit, float upper_limit);

float vector_mean(std::vector<float> vec);
float vector_abs_mean(std::vector<float> vec);
float vector_stdev(std::vector<float> vec);
float vector_max(std::vector<float> vec);

// from mpx-util.h
struct event_pair
{
  cudaEvent_t start;
  cudaEvent_t end;
};

// from mpx-util.h
inline void check_cuda_error(char *message)
{
  cudaThreadSynchronize();
  cudaError_t error = cudaGetLastError();
  if(error != cudaSuccess)
  {
    printf("CUDA error after %s: %s\n", message, cudaGetErrorString(error));
  }
}

inline void start_timer(event_pair * p)
{
  cudaEventCreate(&p->start);
  cudaEventCreate(&p->end);
  cudaEventRecord(p->start, 0);
}

inline void stop_timer(event_pair * p, char * kernel_name)
{
  cudaEventRecord(p->end, 0);
  cudaEventSynchronize(p->end);

  float elapsed_time;
  cudaEventElapsedTime(&elapsed_time, p->start, p->end);
  printf("%s took %.1f ms\n",kernel_name, elapsed_time);
  cudaEventDestroy(p->start);
  cudaEventDestroy(p->end);
}

inline float elapsed_time(event_pair * p)
{
  cudaEventRecord(p->end, 0);
  cudaEventSynchronize(p->end);

  float elapsed_time_;
  cudaEventElapsedTime(&elapsed_time_, p->start, p->end);
  cudaEventDestroy(p->start);
  cudaEventDestroy(p->end);

  return elapsed_time_;
}

#endif // UTIL_H
